unit WDosWatchdogs;

interface

{$IFDEF DESIGNPACKAGE}
uses
  SysUtils, Messages, Classes, Forms;
{$ELSE}
uses
  WDosMessages, WDosSysUtils, WDosClasses, WDosForms, WDosInterrupts, WDosPorts;
{$ENDIF}

const
  WM_WATCHDOG = WM_USER +1;

type
  {$IFDEF DESIGNPACKAGE}
  DpmiPmVector = Int64;
  {$ENDIF}

  TWatchMode = (wmReset, wmInterrupt);

  TCustomWatchdog = class(TComponent)
  private
    FEnabled: Boolean;
    FStreamedEnabled: Boolean;
    FInterval: Integer;
    FModel: string;
    FOnTimeOut: TNotifyEvent;
    FOnTimeOutInterrupt: TNotifyEvent;
    FMode: TWatchMode;
    procedure SetEnabled(const Value: Boolean);
    procedure SetInterval(const Value: Integer);
    procedure SetMode(const Value: TWatchMode);
    procedure SetModel(const Value: string);
    procedure WMWatchdog(var Msg: TMessage); message WM_WATCHDOG;
  protected
    procedure CheckLock;
    procedure SetModelText(Value: string);
    procedure DoSetMode(Value: TWatchMode); virtual;
    procedure DoSetInterval(var Value: Integer); virtual;
    procedure DoSetEnabled(Value: Boolean); virtual; abstract;
    procedure DoRefresh; virtual; abstract;
    procedure DoInterrupt; virtual;
    procedure DoWatchdog; virtual;
    procedure Loaded; override;
    property Interval: Integer read FInterval write SetInterval;
    property Mode: TWatchMode read FMode write SetMode;
    property OnTimeOut: TNotifyEvent read FOnTimeOut write FOnTimeOut;
    property OnTimeOutInterrupt: TNotifyEvent read FOnTimeOutInterrupt
        write FOnTimeOutInterrupt;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh;
    property Model: string read FModel write SetModel;
    property Enabled: Boolean read FEnabled write SetEnabled;
  end;

  EWatchdogException = class(Exception);

  TIntervalMode = (im03, im06, im12, im18, im24, im30, im36, im42);

const
  IntervalDefault = 3000;

{ watchdog for Acrosser Model AR-B1576/AR-B1577 }
{ www.acrosser.com }

type
  TwdxArWatchDog = class(TCustomWatchdog)
  private
    FWatchdogData: Byte;
    FIntervalMode: TIntervalMode;
    FInt77: DpmiPmVector;
    function CalcData: Byte;
  protected
    procedure DoSetEnabled(Value: Boolean); override;
    procedure DoSetInterval(var Value: Integer); override;
    procedure DoRefresh; override;
    procedure DoInterrupt; override;
    procedure DoEnable;
    procedure DoDisable;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Model;
    property Enabled;
    property Interval default IntervalDefault;
    property Mode;
    property OnTimeOut;
    property OnTimeOutInterrupt;
  end;

implementation

var
  WatchdogSingleton: Boolean;
  Watchdog: TCustomWatchdog;

{$IFNDEF DESIGNPACKAGE}
procedure WatchdogInterrupt;
begin
  if Assigned(Watchdog) then
    Watchdog.DoInterrupt;
  { enable interrupts }
  Port[$A0]:=$20;
  Port[$20]:=$20;
end;
{$ENDIF}

{ TCustomWatchdog }

constructor TCustomWatchdog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if WatchdogSingleton then
    raise EWatchdogException.Create('Watchdog-Instanz existiert bereits');
  WatchdogSingleton :=True;
  Watchdog :=Self;
  SetModelText('CustomWatchdog');
  {$IFNDEF DESIGNPACKAGE}
  if Assigned(Application) then
    Application.RegisterMessageObject(Self);
  {$ENDIF}
end;

destructor TCustomWatchdog.Destroy;
begin
  Enabled :=False;
  {$IFNDEF DESIGNPACKAGE}
  if Assigned(Application) then
    Application.UnregisterMessageObject(Self);
  {$ENDIF}
  WatchdogSingleton :=False;
  Watchdog :=nil;
  inherited Destroy;
end;

procedure TCustomWatchdog.CheckLock;
begin
  if FEnabled then
    raise EWatchdogException.Create('Eigenschaft kann nicht im aktiven Modus gendert werden');
end;

procedure TCustomWatchdog.DoSetInterval(var Value: Integer);
begin
  { do nothing }
end;

procedure TCustomWatchdog.DoSetMode(Value: TWatchMode);
begin
  { do nothing }
end;

procedure TCustomWatchdog.Refresh;
begin
  DoRefresh;
end;

procedure TCustomWatchdog.SetEnabled(const Value: Boolean);
begin
  if csLoading in ComponentState then
    FStreamedEnabled :=Value
  else
    if FEnabled <>Value then
    begin
      DoSetEnabled(Value);
      FEnabled := Value;
    end;
end;

procedure TCustomWatchdog.SetInterval(const Value: Integer);
var
  lInterval: Integer;
begin
  if FInterval <>Value then
  begin
    CheckLock;
    lInterval :=Value;
    DoSetInterval(lInterval);
    FInterval := lInterval;
  end;
end;

procedure TCustomWatchdog.SetMode(const Value: TWatchMode);
begin
  if FMode <>Value then
  begin
    CheckLock;
    DoSetMode(Value);
    FMode := Value;
  end;
end;

procedure TCustomWatchdog.SetModel(const Value: string);
begin
  { dummy function }
end;

procedure TCustomWatchdog.SetModelText(Value: string);
begin
  if FModel <>Value then FModel := Value;
end;

procedure TCustomWatchdog.DoInterrupt;
begin
  if Assigned(FOnTimeOutInterrupt) then OnTimeOutInterrupt(Self);
  {$IFNDEF DESIGNPACKAGE}
  if Assigned(Application) then
    Application.PostMessage(Self, WM_WATCHDOG, 0, 0);
  {$ENDIF}
end;

procedure TCustomWatchdog.WMWatchdog(var Msg: TMessage);
begin
  DoWatchdog;
end;

procedure TCustomWatchdog.DoWatchdog;
begin
  if Assigned(FOnTimeOut) then OnTimeOut(Self);
end;

procedure TCustomWatchdog.Loaded;
begin
  inherited Loaded;
  try
    Enabled :=FStreamedEnabled;
  except
    { let Delphi handle exceptions in design modus }
    if csDesigning in ComponentState then
      Application.HandleException(Self)
    else
      raise;
  end;
end;

{ TwdxArWatchDog }

const
  WatchAddr = $76;
  IntControl  =$A1; { 2nd interrupt controller }

constructor TwdxArWatchDog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetModelText('Acrosser AR-B1576/77');
  Interval :=IntervalDefault;
end;

procedure TwdxArWatchDog.DoRefresh;
begin
  {$IFNDEF DESIGNPACKAGE}
  if Enabled then Port[WatchAddr] :=FWatchdogData;
  {$ENDIF}
end;

procedure TwdxArWatchDog.DoSetEnabled(Value: Boolean);
begin
  if Value then DoEnable else DoDisable;
end;

procedure TwdxArWatchDog.DoDisable;

  {$IFNDEF DESIGNPACKAGE}
  var
    Reg: Byte;
  {$ENDIF}

begin
  {$IFNDEF DESIGNPACKAGE}
  if Mode =wmInterrupt then
  begin
    { disable RTC interrupt on interrupt controller }
    Reg :=Port[IntControl];
    Reg :=Reg or $80;
    Port[IntControl] :=Reg;
    { restore old interrupt vector }
    SetIntVec($77, FInt77);
  end;
  Port[WatchAddr] :=0;
  {$ENDIF}
end;

procedure TwdxArWatchDog.DoEnable;

  {$IFNDEF DESIGNPACKAGE}
  var
    Reg: Byte;
  {$ENDIF}

begin
  FWatchdogData :=CalcData;
  {$IFNDEF DESIGNPACKAGE}
  Port[WatchAddr] :=FWatchdogData;
  {$ENDIF}
  if Mode = wmInterrupt then
  begin
   {$IFDEF DESIGNPACKAGE}
   FInt77 :=0;
   {$ELSE}
   { redirect interrupt vector on own procedure }
   GetIntVec($77, FInt77);
   SetIntProc($77, @WatchdogInterrupt);
   { enable RTC interrupt at interrupt controller }
   Reg :=Port[IntControl];
   Reg :=Reg and $7F;
   Port[IntControl] :=Reg;
   {$ENDIF}
  end;
end;

procedure TwdxArWatchDog.DoSetInterval(var Value: Integer);

  const
    IntervalVal: array[TIntervalMode] of Integer =
      (3000, 6000, 12000, 18000, 24000, 30000, 36000, 42000);

var
  I: TIntervalMode;
begin
  if Value <IntervalVal[Low(TIntervalMode)] then
    Value :=IntervalVal[Low(TIntervalMode)];
  for I :=Low(TIntervalMode) to High(TIntervalMode) do
    if Value >= IntervalVal[I] then
      FIntervalMode :=I
    else Break;
  Value :=IntervalVal[FIntervalMode];
end;

function TwdxArWatchDog.CalcData: Byte;

  const
    ModeVal: array[TWatchMode] of Byte = ($80, $C0);

begin
  Result :=ModeVal[Mode] or Byte(FIntervalMode);
end;

procedure TwdxArWatchDog.DoInterrupt;
begin
  inherited DoInterrupt;
  Refresh;
end;

end.
